@@ -33,6 +33,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -46,7 +47,7 @@ typedef enum {
 	CALLBACK_GET_KEYRING_INFO,
 	CALLBACK_GET_ITEM_INFO,
 	CALLBACK_GET_ATTRIBUTES,
-	CALLBACK_GET_ACL,
+	CALLBACK_GET_ACL
 } KeyringCallbackType;
 
 typedef enum {
@@ -581,6 +582,27 @@ gnome_keyring_set_default_keyring (const char                             *keyri
 	return op;
 }
 
+GnomeKeyringResult 
+gnome_keyring_set_default_keyring_sync (const char *keyring)
+{
+	GString *send;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_op_string (send,
+						   GNOME_KEYRING_OP_SET_DEFAULT_KEYRING,
+						   keyring)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+
+	return res;
+}
+
 gpointer
 gnome_keyring_get_default_keyring (GnomeKeyringOperationGetStringCallback  callback,
 				   gpointer                                data,
@@ -602,6 +624,41 @@ gnome_keyring_get_default_keyring (GnomeKeyringOperationGetStringCallback  callb
 	return op;
 }
 
+GnomeKeyringResult 
+gnome_keyring_get_default_keyring_sync (char **keyring)
+{
+	GString *send;
+	GString *receive;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+	
+	*keyring = NULL;
+
+	if (!gnome_keyring_proto_encode_op_only (send,
+						 GNOME_KEYRING_OP_GET_DEFAULT_KEYRING)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+
+	receive = g_string_new (NULL);
+
+	res = run_sync_operation (send, receive);
+	g_string_free (send, TRUE);
+	if (res != GNOME_KEYRING_RESULT_OK) {
+		g_string_free (receive, TRUE);
+		return res;
+	}
+
+	if (!gnome_keyring_proto_decode_result_string_reply (receive, &res, keyring)) {
+		g_string_free (receive, TRUE);
+		return GNOME_KEYRING_RESULT_IO_ERROR;
+	}
+	g_string_free (receive, TRUE);
+	
+	return res;
+}
+
 static void
 gnome_keyring_list_keyring_names_reply (GnomeKeyringOperation *op)
 {
@@ -641,6 +698,40 @@ gnome_keyring_list_keyring_names  (GnomeKeyringOperationGetListCallback    callb
 	return op;
 }
 
+GnomeKeyringResult 
+gnome_keyring_list_keyring_names_sync (GList **keyrings)
+{
+	GString *send;
+	GString *receive;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+	
+	*keyrings = NULL;
+
+	if (!gnome_keyring_proto_encode_op_only (send,
+						 GNOME_KEYRING_OP_LIST_KEYRINGS)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+
+	receive = g_string_new (NULL);
+
+	res = run_sync_operation (send, receive);
+	g_string_free (send, TRUE);
+	if (res != GNOME_KEYRING_RESULT_OK) {
+		g_string_free (receive, TRUE);
+		return res;
+	}
+
+	if (!gnome_keyring_proto_decode_result_string_list_reply (receive, &res, keyrings)) {
+		g_string_free (receive, TRUE);
+		return GNOME_KEYRING_RESULT_IO_ERROR;
+	}
+	g_string_free (receive, TRUE);
+	
+	return res;
+}
  
 gpointer
 gnome_keyring_lock_all (GnomeKeyringOperationDoneCallback       callback,
@@ -663,6 +754,26 @@ gnome_keyring_lock_all (GnomeKeyringOperationDoneCallback       callback,
 	return op;
 }
 
+GnomeKeyringResult 
+gnome_keyring_lock_all_sync (void)
+{
+	GString *send;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+
+	if (!gnome_keyring_proto_encode_op_only (send,
+						 GNOME_KEYRING_OP_LOCK_ALL)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+}
+
 
 
 /* NULL password means ask user */
@@ -691,6 +802,28 @@ gnome_keyring_create (const char                                  *keyring_name,
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_create_sync (const char *keyring_name,
+			   const char *password)
+{
+	GString *send;
+	GnomeKeyringResult res;
+	
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_op_string_string (send,
+							  GNOME_KEYRING_OP_CREATE_KEYRING,
+							  keyring_name, password)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+}
+
 gpointer
 gnome_keyring_unlock (const char                                  *keyring,
 		      const char                                  *password,
@@ -716,6 +849,28 @@ gnome_keyring_unlock (const char                                  *keyring,
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_unlock_sync (const char *keyring,
+			   const char *password)
+{
+	GString *send;
+	GnomeKeyringResult res;
+	
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_op_string_string (send,
+							  GNOME_KEYRING_OP_UNLOCK_KEYRING,
+							  keyring, password)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+}
+
 gpointer
 gnome_keyring_lock (const char                                  *keyring,
 		    GnomeKeyringOperationDoneCallback            callback,
@@ -740,6 +895,27 @@ gnome_keyring_lock (const char                                  *keyring,
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_lock_sync (const char *keyring)
+{
+	GString *send;
+	GnomeKeyringResult res;
+	
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_op_string (send,
+						   GNOME_KEYRING_OP_LOCK_KEYRING,
+						   keyring)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+}
+
 gpointer
 gnome_keyring_delete (const char                                  *keyring,
 		      GnomeKeyringOperationDoneCallback            callback,
@@ -764,6 +940,27 @@ gnome_keyring_delete (const char                                  *keyring,
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_delete_sync (const char *keyring)
+{
+	GString *send;
+	GnomeKeyringResult res;
+	
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_op_string (send,
+						   GNOME_KEYRING_OP_DELETE_KEYRING,
+						   keyring)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+}
+
 static void
 gnome_keyring_get_keyring_info_reply (GnomeKeyringOperation *op)
 {
@@ -805,6 +1002,42 @@ gnome_keyring_get_info (const char                                  *keyring,
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_get_info_sync (const char        *keyring,
+			     GnomeKeyringInfo **info)
+{
+	GString *send;
+	GString *receive;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+	
+	*info = NULL;
+
+	if (!gnome_keyring_proto_encode_op_only (send,
+						 GNOME_KEYRING_OP_GET_KEYRING_INFO)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+
+	receive = g_string_new (NULL);
+
+	res = run_sync_operation (send, receive);
+	g_string_free (send, TRUE);
+	if (res != GNOME_KEYRING_RESULT_OK) {
+		g_string_free (receive, TRUE);
+		return res;
+	}
+
+	if (!gnome_keyring_proto_decode_get_keyring_info_reply (receive, &res, info)) {
+		g_string_free (receive, TRUE);
+		return GNOME_KEYRING_RESULT_IO_ERROR;
+	}
+	g_string_free (receive, TRUE);
+	
+	return res;
+}
+
 gpointer
 gnome_keyring_set_info (const char                                  *keyring,
 			GnomeKeyringInfo                            *info,
@@ -829,6 +1062,27 @@ gnome_keyring_set_info (const char                                  *keyring,
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_set_info_sync (const char       *keyring,
+			     GnomeKeyringInfo *info)
+{
+	GString *send;
+	GnomeKeyringResult res;
+	
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_set_keyring_info (send,
+							  keyring, info)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+}
+
 static void
 gnome_keyring_list_item_ids_reply (GnomeKeyringOperation *op)
 {
@@ -869,6 +1123,43 @@ gnome_keyring_list_item_ids (const char                                  *keyrin
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_list_item_ids_sync (const char  *keyring,
+				  GList      **ids)
+{
+	GString *send;
+	GString *receive;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+	
+	*ids = NULL;
+
+	if (!gnome_keyring_proto_encode_op_string (send,
+						   GNOME_KEYRING_OP_LIST_ITEMS,
+						   keyring)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+
+	receive = g_string_new (NULL);
+
+	res = run_sync_operation (send, receive);
+	g_string_free (send, TRUE);
+	if (res != GNOME_KEYRING_RESULT_OK) {
+		g_string_free (receive, TRUE);
+		return res;
+	}
+
+	if (!gnome_keyring_proto_decode_result_int_list_reply (receive, &res, ids)) {
+		g_string_free (receive, TRUE);
+		return GNOME_KEYRING_RESULT_IO_ERROR;
+	}
+	g_string_free (receive, TRUE);
+	
+	return res;
+}
+
 void
 gnome_keyring_info_set_lock_on_idle (GnomeKeyringInfo *keyring_info,
 				     gboolean          value)
@@ -1186,6 +1477,29 @@ gnome_keyring_item_delete (const char                                 *keyring,
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_item_delete_sync (const char *keyring,
+				guint32     id)
+{
+	GString *send;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_op_string_int (send,
+						       GNOME_KEYRING_OP_DELETE_ITEM,
+						       keyring,
+						       id)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+
+	return res;
+}
+
 static void
 gnome_keyring_get_item_info_reply (GnomeKeyringOperation *op)
 {
@@ -1228,6 +1542,44 @@ gnome_keyring_item_get_info (const char                                 *keyring
 	return op;
 }
 
+GnomeKeyringResult 
+gnome_keyring_item_get_info_sync (const char            *keyring,
+				  guint32                id,
+				  GnomeKeyringItemInfo **info)
+{
+	GString *send;
+	GString *receive;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+
+	*info = NULL;
+	
+	if (!gnome_keyring_proto_encode_op_string_int (send, 
+						       GNOME_KEYRING_OP_GET_ITEM_INFO,
+						       keyring, id)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	receive = g_string_new (NULL);
+
+	res = run_sync_operation (send, receive);
+	g_string_free (send, TRUE);
+	if (res != GNOME_KEYRING_RESULT_OK) {
+		g_string_free (receive, TRUE);
+		return res;
+	}
+	
+	if (!gnome_keyring_proto_decode_get_item_info_reply (receive, &res, info)) {
+		g_string_free (receive, TRUE);
+		return GNOME_KEYRING_RESULT_IO_ERROR;
+	}
+	g_string_free (receive, TRUE);
+	
+	return res;
+}
+
 gpointer
 gnome_keyring_item_set_info (const char                                 *keyring,
 			     guint32                                     id,
@@ -1253,6 +1605,28 @@ gnome_keyring_item_set_info (const char                                 *keyring
 	return op;
 }
 
+GnomeKeyringResult 
+gnome_keyring_item_set_info_sync (const char           *keyring,
+				  guint32               id,
+				  GnomeKeyringItemInfo *info)
+{
+	GString *send;
+	GnomeKeyringResult res;
+	
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_set_item_info (send,
+						       keyring, id, info)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+}
+
 static void
 gnome_keyring_get_attributes_reply (GnomeKeyringOperation *op)
 {
@@ -1313,6 +1687,44 @@ gnome_keyring_item_get_attributes (const char                                 *k
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_item_get_attributes_sync (const char                 *keyring,
+					guint32                     id,
+					GnomeKeyringAttributeList **attributes)
+{
+	GString *send;
+	GString *receive;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+
+	*attributes = NULL;
+	
+	if (!gnome_keyring_proto_encode_op_string_int (send, 
+						       GNOME_KEYRING_OP_GET_ITEM_ATTRIBUTES,
+						       keyring, id)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	receive = g_string_new (NULL);
+
+	res = run_sync_operation (send, receive);
+	g_string_free (send, TRUE);
+	if (res != GNOME_KEYRING_RESULT_OK) {
+		g_string_free (receive, TRUE);
+		return res;
+	}
+	
+	if (!gnome_keyring_proto_decode_get_attributes_reply (receive, &res, attributes)) {
+		g_string_free (receive, TRUE);
+		return GNOME_KEYRING_RESULT_IO_ERROR;
+	}
+	g_string_free (receive, TRUE);
+	
+	return res;
+}
+
 gpointer
 gnome_keyring_item_set_attributes (const char                                 *keyring,
 				   guint32                                     id,
@@ -1339,6 +1751,30 @@ gnome_keyring_item_set_attributes (const char                                 *k
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_item_set_attributes_sync (const char                *keyring,
+					guint32                    id,
+					GnomeKeyringAttributeList *attributes)
+{
+	GString *send;
+	GnomeKeyringResult res;
+	
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_set_attributes (send,
+							keyring, id, 
+							attributes)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+
+}
+
 gpointer
 gnome_keyring_item_get_acl (const char                                 *keyring,
 			    guint32                                     id,
@@ -1364,6 +1800,44 @@ gnome_keyring_item_get_acl (const char                                 *keyring,
 	return op;
 }
 
+GnomeKeyringResult
+gnome_keyring_item_get_acl_sync (const char  *keyring,
+				 guint32      id,
+				 GList      **acl)
+{
+	GString *send;
+	GString *receive;
+	GnomeKeyringResult res;
+
+	send = g_string_new (NULL);
+
+	*acl = NULL;
+	
+	if (!gnome_keyring_proto_encode_op_string_int (send, 
+						       GNOME_KEYRING_OP_GET_ITEM_ACL,
+						       keyring, id)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	receive = g_string_new (NULL);
+
+	res = run_sync_operation (send, receive);
+	g_string_free (send, TRUE);
+	if (res != GNOME_KEYRING_RESULT_OK) {
+		g_string_free (receive, TRUE);
+		return res;
+	}
+	
+	if (!gnome_keyring_proto_decode_get_acl_reply (receive, &res, acl)) {
+		g_string_free (receive, TRUE);
+		return GNOME_KEYRING_RESULT_IO_ERROR;
+	}
+	g_string_free (receive, TRUE);
+	
+	return res;
+}
+
 gpointer
 gnome_keyring_item_set_acl (const char                                 *keyring,
 			    guint32                                     id,
@@ -1390,6 +1864,29 @@ gnome_keyring_item_set_acl (const char                                 *keyring,
 	return op;
 }
 
+GnomeKeyringResult 
+gnome_keyring_item_set_acl_sync (const char *keyring,
+				 guint32     id,
+				 GList      *acl)
+{
+	GString *send;
+	GnomeKeyringResult res;
+	
+	send = g_string_new (NULL);
+	
+	if (!gnome_keyring_proto_encode_set_acl (send,
+						 keyring, id, 
+						 acl)) {
+		g_string_free (send, TRUE);
+		return GNOME_KEYRING_RESULT_BAD_ARGUMENTS;
+	}
+	
+	res = run_sync_operation (send, NULL);
+	g_string_free (send, TRUE);
+	
+	return res;
+}
+
 GnomeKeyringItemType
 gnome_keyring_item_info_get_type (GnomeKeyringItemInfo *item_info)
 {
